package com.supervision.springaidemo.controller;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.json.JSONUtil;
import cn.hutool.poi.excel.ExcelReader;
import cn.hutool.poi.excel.ExcelUtil;
import cn.hutool.poi.excel.ExcelWriter;
import com.supervision.springaidemo.domain.VpAskTemplateQuestionLibrary;
import com.supervision.springaidemo.domain.VpCommonDic;
import com.supervision.springaidemo.service.VpAskTemplateQuestionLibraryService;
import com.supervision.springaidemo.service.VpCommonDicService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.ai.document.Document;
import org.springframework.ai.embedding.EmbeddingClient;
import org.springframework.ai.ollama.OllamaChatClient;
import org.springframework.ai.ollama.OllamaEmbeddingClient;
import org.springframework.ai.ollama.api.OllamaOptions;
import org.springframework.ai.vectorstore.SimpleVectorStore;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.io.File;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Slf4j
@RequestMapping("similarity")
@RestController
public class SimilarityController {

    private final SimpleVectorStore store;


    private final OllamaChatClient chatClient;

    @Autowired
    private VpCommonDicService vpCommonDicService;

    @Autowired
    private VpAskTemplateQuestionLibraryService libraryService;

    @Autowired
    public SimilarityController(OllamaEmbeddingClient ollamaEmbeddingClient, OllamaChatClient chatClient) {
        store = new SimpleVectorStore(ollamaEmbeddingClient.withDefaultOptions(OllamaOptions.create().withModel("llama3-chinese:8b")));
        this.chatClient = chatClient;
//        File file = FileUtil.file("/Users/flevance/Java/spring-ai-demo/spring-ai-demo/data/storeData.json");
//        if (FileUtil.isNotEmpty(file)) {
//            store.load(file);
//        }
    }

    @GetMapping("init")
    public void init() {
        // 从数据库中加载向量数据
        List<VpAskTemplateQuestionLibrary> list = libraryService.lambdaQuery().list();
        for (VpAskTemplateQuestionLibrary vpAskTemplateQuestionLibrary : list) {
            String questionJsonStr = vpAskTemplateQuestionLibrary.getQuestion();
            List<String> questionList = JSONUtil.toList(questionJsonStr, String.class);
            questionList.add(vpAskTemplateQuestionLibrary.getDescription());
            for (String question : questionList) {
                Map<String, Object> meta = MapUtil.of("dictId", vpAskTemplateQuestionLibrary.getDictId());
                meta.put("id", vpAskTemplateQuestionLibrary.getId());
                Document document = new Document(question, meta);
                store.add(ListUtil.of(document));
                log.info("{}已添加到向量数据库", question);
            }
        }
        log.info("向量化完成");
        // 加载完成之后,保存到本地
        store.save(FileUtil.newFile("/Users/flevance/Java/spring-ai-demo/spring-ai-demo/data/storeData.json"));
        log.info("保存完成");
    }

    @GetMapping("similarity")
    public void similarity(String userQuestion) {
        List<Document> documents = store.similaritySearch(userQuestion);
        for (Document document : documents) {
            log.info("{}", document.getContent());
        }
    }

    @GetMapping("itemCheck")
    public void itemCheck() {
        File file = FileUtil.file("/Users/flevance/Desktop/虚拟病人/副本副本医生临床问诊问题收集v1.xlsx");
        ExcelReader reader = ExcelUtil.getReader(file, 1);
        ExcelWriter writer = ExcelUtil.getWriter(file, "match");
        List<VpCommonDic> list = vpCommonDicService.lambdaQuery().like(VpCommonDic::getNameZhPath, "/").eq(VpCommonDic::getGroupCode, "AQT").list();
        Set<String> databaseExistSet = new HashSet<>();
        for (VpCommonDic vpCommonDic : list) {
            databaseExistSet.add(vpCommonDic.getNameZhPath());
        }
        List<List<Object>> objects = reader.read(1);
        for (int i = 0; i < objects.size(); i++) {
            List<Object> list1 = objects.get(i);
            String s = (String) list1.get(0);
            String s1 = (String) list1.get(1);
            // 如果不存在,则记录
            if (!databaseExistSet.contains(s + "/" + s1)) {
                writer.writeCellValue(1, i + 1, "不存在");
            } else {
                writer.writeCellValue(1, i + 1, "存在");
            }
        }
        writer.flush();

    }

    @GetMapping("similarityBatch")
    public void similarityBatch() {
        init();
        Map<Long, VpCommonDic> dicMap = vpCommonDicService.list().stream().collect(Collectors.toMap(VpCommonDic::getId, Function.identity()));
        // 从Excel中获取数据
        File file = FileUtil.file("/Users/flevance/Desktop/虚拟病人/副本副本医生临床问诊问题收集v1.xlsx");
        ExcelReader reader = ExcelUtil.getReader(file, 2);
        ExcelWriter writer = ExcelUtil.getWriter(file, "result");

        List<Object> objects = reader.readColumn(2, 1);
        for (int i = 0; i < objects.size(); i++) {
            int rowIndex = i + 1;
            String question = objects.get(i).toString();
            try {
                List<Document> documents = store.similaritySearch(question);
                Optional<Document> first = documents.stream().findFirst();
                if (first.isPresent()) {
                    Map<String, Object> meta = first.get().getMetadata();
                    String dictId = meta.get("dictId").toString();
                    VpCommonDic vpCommonDic = dicMap.get(Long.parseLong(dictId));
                    writer.writeCellValue(8, rowIndex, question);
                    writer.writeCellValue(9, rowIndex, "");
                    writer.writeCellValue(10, rowIndex, vpCommonDic.getNameZhPath());
                    writer.writeCellValue(11, rowIndex, dictId);
                }
            } catch (Exception e) {
                log.error("出现错误", e);
            }
            log.info("{}匹配完成.", question);
        }
        writer.flush();

    }
}
